@@ -9,10 +9,11 @@ from .molecule import packmol
 from .dft.siesta import siesta_md # siesta_opt 
 from .dft.qe import qemd # qeopt  
 from .dft.SinglePointEnergy import SinglePointEnergies
+from .dft.CheckEmol import check_emol
 from .md.gulp import nvt as gulp_nvt
 from .data.mdtodata import MDtoData
 from .data.prep_data import prep_data
-from .reaxfflib import read_lib
+#from .reaxfflib import read_ffield
 import tensorflow as tf
 from .md.irmd import IRMD
 from .trainer import train_reax
@@ -61,7 +62,7 @@ class LearningMachine(object):
                colFrame=50,colmin=3,
                mom_step=100,
                mpopt=[1,1,1,1], messages=1,
-               optword='nocoul',cons=None,
+               optword='nocoul',cons=None,clip={},
                T=300,# Tmax=1000,
                label=None,
                convergence=0.01,nconvergence=3,
@@ -77,19 +78,20 @@ class LearningMachine(object):
                bo_layer=[2,1],mf_layer=[9,2],be_layer=[6,1],vdw_layer=None,#[6,1],
                be_univeral_nn='all',bo_univeral_nn='all',mf_univeral_nn='all',vdw_univeral_nn='all',
                BOFunction=0,EnergyFunction=1,MessageFunction=1,VdwFunction=1,
-               #spv_bm=False,bom={'others':1.0},
                spv_be=False,beup={},belo={},
-               spv_bo=False,boc={},# bolo={},
+               spv_bo=False,#boc={},# bolo={},
                spv_vdw=False,vup={},vlo={},
                spv_ang=False,lambda_ang=0.02,
                weight={'others':2.0},
                lambda_bd=10000.0,lambda_reg=0.0001, # regularize=True,
                lambda_me=0.1,learning_rate=0.0001,
                writelib=5000,ffield='ffield.json',
-               dft='siesta',kpts=(1,1,1),xcf='VDW',xca='DRSLL',basistype='DZP'):
+               dft='siesta',kpts=(1,1,1),xcf='VDW',xca='DRSLL',basistype='DZP',
+               **kwargs):
       ''' max_batch: max number of batch 
               batch: batch size
       '''
+      self.kwargs         = kwargs
       self.initConfig     = initConfig
       self.label          = initConfig.split('.')[0]
       self.aidir          = 'aimd_' + self.label
@@ -136,6 +138,7 @@ class LearningMachine(object):
       self.MinMDstep      = MinMDstep
       self.mpopt          = mpopt
       self.optword        = optword
+      self.clip           = clip
       self.messages       = messages
       self.mdInc          = mdInc       # MD step increase factor
       self.ffield         = ffield
@@ -154,7 +157,7 @@ class LearningMachine(object):
       self.vdw_univeral_nn= vdw_univeral_nn
       self.beup           = beup
       self.belo           = belo
-      self.boc            = boc
+      #self.boc            = boc
       #self.bolo          = bolo
       self.vup            = vup
       self.vlo            = vlo
@@ -254,6 +257,7 @@ class LearningMachine(object):
       zmat_variable  = None
       zmatopt        = None
       zmats          = None
+      images         = None
       zvlo,zvhi      = 0.0,0.0
       u_zvlo,u_zvhi  = 0.0,0.0
 
@@ -268,7 +272,9 @@ class LearningMachine(object):
           if exists(run_dir):
              iter_ += 1
              continue
-          if isfile('md.traj'): 
+          if images is not None:
+             atoms = images[-1]   
+          elif isfile('md.traj'): 
              atoms = read('md.traj',index=-1)
              consistance = self.check_atoms(atoms)
              assert consistance,'-  The species or cell parameter in md.traj is not consistant with initial configuration!'
@@ -324,8 +330,9 @@ class LearningMachine(object):
           else:
              dft_step = int(mdsteps/self.colFrame)+1
 
+          
           e_aimd,eml_,dEmax_,d2Emax_,LabelDataLog = self.aimd(atoms,cwd,run_dir,dft_step,learnWay)
-          e_siesta.append(e_aimd[0])                                   # get siesta results              
+          e_siesta.append(e_aimd[0])                                # get siesta results              
                                                                        # start training  
           trajs_ = prep_data(label=self.label,direcs=data_dir,
                              split_batch=self.batch,max_batch=self.max_batch,
@@ -333,6 +340,7 @@ class LearningMachine(object):
           with open('training.log','w') as fe:                         # 
                fe.write('## training loss and accuries\n')             #
           trajs_.update(self.dataset)                                  #
+          check_emol(trajs_)
 
           self.load_config()
           tf.compat.v1.disable_eager_execution()                       # training untill
@@ -347,7 +355,7 @@ class LearningMachine(object):
                                                      step=self.step,writelib=self.writelib,
                                                      batch=self.batch,
                                                      convergence=self.accCriteria,
-                                                     cons=self.cons,
+                                                     cons=self.cons,clip=self.clip,
                                                      lossConvergence=self.lossCriteria,
                                                      nn=self.nn,vdwnn=self.vdwnn,
                                                      mpopt=self.mpopt,messages=self.messages,
@@ -366,7 +374,7 @@ class LearningMachine(object):
                                                      MessageFunction=self.MessageFunction,
                                                      # bore=self.bore, # bom=self.bom,
                                                      spv_be=self.spv_be,belo=self.belo,beup=self.beup,
-                                                     spv_bo=self.spv_bo,boc=self.boc,#boup=self.boup,
+                                                     spv_bo=self.spv_bo,#boc=self.boc,#boup=self.boup,
                                                      spv_vdw=self.spv_vdw,vlo=self.vlo,vup=self.vup,
                                                      spv_ang=self.spv_ang,lambda_ang=self.lambda_ang,
                                                      lambda_me=self.lambda_me,weight=self.weight,
@@ -391,9 +399,9 @@ class LearningMachine(object):
           
           if self.CheckZmat:
              if not zmatopt is None:
-                optlog = self.a.get_optimal_zv(atoms,zmatopt,self.initConfig)
+                optlog,_ = self.a.get_optimal_zv(atoms,zmatopt,optgen=self.initConfig)
              if mdsteps>10:
-                uncertainty_zv,u_zvlo,u_zvhi = self.a.get_zmat_info(zmats)
+                uncertainty_zv,u_zvlo,u_zvhi = self.a.get_zmat_uncertainty(atoms)
 
           if mdsteps<10: mdsteps = 10
           zmats   = None
@@ -405,14 +413,14 @@ class LearningMachine(object):
                 i = self.freeatoms[ms]
                 self.a.pes(i,atoms,nbin=dft_step,dr=0.1,traj='md.traj')
              if ms == len(self.freeatoms) -1:
-             	self.learnWay=3
+                self.learnWay=3
           else:
              if Deformed>=1.0:
                 images,relaxlog = self.a.zmat_relax(atoms=atoms,zmat_variable=zmat_variable,nbin=mdsteps,
                                                     zvlo=zvlo,zvhi=zvhi,traj='md.traj')
              elif not uncertainty_zv is None:
                 images,relaxlog = self.a.zmat_relax(atoms=atoms,zmat_variable=uncertainty_zv,nbin=mdsteps,
-                                                    zvlo=u_zvlo,zvhi=u_zvhi,traj='md.traj')
+                                                    zvlo=u_zvlo,zvhi=u_zvhi,traj='md.traj',reset=True)
              else:
                 e_gmd,mdsteps,Deformed,zmats,zmat_variable,zvlo,zvhi = self.mlmd(cwd,run_dir,atoms, 10000,
                                                 iter_,learnWay,beta=self.beta,learnpair=self.learnpair,
@@ -533,15 +541,14 @@ class LearningMachine(object):
          if self.dft=='siesta':
             images = siesta_md(label=self.label,ncpu=self.ncpu,T=self.T,dt=self.dt_aimd,us='F',tstep=tstep,
                                atoms=atoms,FreeAtoms=self.freeatoms,
-                               xcf=self.xcf,xca=self.xca,basistype=self.basistype)
+                               xcf=self.xcf,xca=self.xca,basistype=self.basistype,**self.kwargs)
             # images = siesta_opt(atoms=atoms,label=self.label,ncpu=self.ncpu,VariableCell=False,us='F',
             #                     tstep=tstep,gen=cwd+'/'+gen,
             #                     xcf=self.xcf,xca=self.xca,basistype=self.basistype)
             ind_   = 'Molecular Dynamics Simulation'
          elif self.dft=='qe':
             images = qemd(label=self.label,ncpu=self.ncpu,T=self.T,dt=self.dt_aimd,tstep=tstep,
-                          atoms=atoms,
-                          kpts=self.kpts)         ## geomentry optimize
+                          atoms=atoms,kpts=self.kpts,**self.kwargs)      ## geomentry optimize
             # images = qeopt(atoms=atoms,label=self.label,ncpu=self.ncpu,
             #                gen=cwd+'/'+gen,
             #                kpts=self.kpts)         ## geomentry optimize
@@ -556,10 +563,10 @@ class LearningMachine(object):
          # print(' * files in this dir \n',listdir())
          E,E_,dEmax,d2Emax,ind_ = SinglePointEnergies('md.traj',label=self.label,EngTole=self.EngTole,
                                                  frame=tstep,select=True,
-                                                 dE=self.dEtole, colmin=self.colmin,# d2E=self.d2Etole,
+                                                 dE=self.dEtole, colmin=self.colmin,
                                                  dft=self.dft,kpts=self.kpts,
                                                  xcf=self.xcf,xca=self.xca,basistype=self.basistype,
-                                                 cpu=self.ncpu)
+                                                 cpu=self.ncpu,**self.kwargs)
          eaimd = E
          emlmd = E_
       else:
