@@ -25,10 +25,15 @@ def single_point(atoms,id=0,kpts=(1,1,1),
 
     images  = read_espresso_out('pw.out')
     system('cp pw.out pw-{:d}.out'.format(id))
+    system('rm -r pwscf.*')
     
     atoms_  = None
     for atoms_ in images:
+        #try:
         e = atoms_.get_potential_energy()
+        # except:
+        #    print('- QE SCF calculation is not converged.')
+        #    return None
     return atoms_
 
 def qemd(atoms=None,gen='poscar.gen',kpts=(1,1,1),
@@ -76,7 +81,6 @@ def pseudopotentials(specie):
 def write_qe_in(atoms,
                 kspacing=None, kpts=None,koffset=(0, 0, 0),
                 calculation=None,          # supported: 'md','relax'
-                forc_conv_thr=0.0001,
                 ion_dynamics='bfgs',
                 nspin=1,fspin=None,**kwargs):
     ''' wtrite QE(PWscf) input'''
@@ -99,8 +103,8 @@ def write_qe_in(atoms,
 
     if any(atoms.get_initial_magnetic_moments()):
         if nspin == 1:
-            # Force spin on
-            nspin = 2
+           # Force spin on
+           nspin = 2
 
     if nspin == 2:
         # Spin on
@@ -138,8 +142,6 @@ def write_qe_in(atoms,
                 '{atom.symbol}{tidx} '
                 '{atom.x:.10f} {atom.y:.10f} {atom.z:.10f}'
                 '{mask}\n'.format(atom=atom, tidx=tidx, mask=mask))
-        #for sp in 
-        
     else:
         for atom in atoms:
             if atom.symbol not in atomic_species:
@@ -174,7 +176,16 @@ def write_qe_in(atoms,
     pwi.append('  outdir          =  \'./\' \n')
     pwi.append('  pseudo_dir      =  \'./\' \n')
     pwi.append('  wfcdir          =  \'./\' \n')
-    pwi.append('  forc_conv_thr   =  {:f}\n'.format(forc_conv_thr))
+    if 'nstep' in kwargs:
+       pwi.append('  nstep           =  {:d} \n'.format(kwargs['nstep']))
+    else:
+       pwi.append('  nstep           =  100 \n')
+    if 'forc_conv_thr' in kwargs:
+       pwi.append('  forc_conv_thr   =  {:f}\n'.format(kwargs['forc_conv_thr']))
+    else:
+       pwi.append('  forc_conv_thr   =  {:f}\n'.format(0.0001))
+    # if 'tprnfor' in kwargs:
+    pwi.append('  tprnfor         =  .TRUE.\n')
     pwi.append('/\n')  # terminate section
     pwi.append('\n')
 
@@ -196,30 +207,42 @@ def write_qe_in(atoms,
     pwi.append('  occupations     =  \'smearing\' \n')
     pwi.append('  smearing        =  \'gauss\' \n') # methfessel-paxton
     pwi.append('  degauss         =  0.2          !defult 0.D0 Ry \n')
-
+    if 'nspin' in kwargs:
+       pwi.append('  nspin         =  {:d} \n'.format(kwargs['nspin']))
+    elif nspin==2:
+       pwi.append('  nspin         =  2 \n')
+       
     if nspin==2:
        for m in mag_str:
            m_ = float(m.split()[-1])
            if abs(m_)>0.00001:
               pwi.append('  '+m+'\n')
     if 'lda_plus_u' in kwargs:
-       pwi.append('  lda_plus_u   =  .true. \n')
+       pwi.append('  lda_plus_u   =  .TRUE. \n')
     if 'hubbard_u' in kwargs:
        hubbard_u = kwargs['hubbard_u']
        for u in hubbard_u:
            pwi.append('  hubbard_u({:d})   =  {:6.4f} \n'.format(u,hubbard_u[u]))
+    if 'input_dft' in kwargs:
+       pwi.append('  input_dft   =  \'{:s}\' \n'.format(kwargs['input_dft']))   
     pwi.append('/\n')  # terminate section
     pwi.append('\n')
 
     pwi.append('&ELECTRONS\n')
+    if 'electron_maxstep' in kwargs:
+       pwi.append('  electron_maxstep =  {:d} \n'.format(kwargs['electron_maxstep']))
+    if 'diagonalization' in kwargs:
+       pwi.append('  diagonalization =  \'{:s}\' \n'.format(kwargs['diagonalization']))
     if 'conv_thr' in kwargs:
-       pwi.append('  conv_thr         =  {:f} \n'.format(kwargs['conv_thr']))
+       pwi.append('  conv_thr         =  {:e} \n'.format(kwargs['conv_thr']))
     else:
-       pwi.append('  conv_thr         =  1.00000e-06           !defult 1.0d-6 \n')
+       pwi.append('  conv_thr         =  1.00000e-6           !defult 1.0d-6 \n')
     if 'mixing_beta' in kwargs:
        pwi.append('  mixing_beta      =  {:f} \n'.format(kwargs['mixing_beta']))
     else:
        pwi.append('  mixing_beta      =  7.00000e-01           !defult 0.7D0 \n')
+    if 'mixing_mode' in kwargs:
+       pwi.append('  mixing_mode      =  \'{:s}\' \n'.format(kwargs['mixing_mode']))
     pwi.append('/\n')  # terminate section
     pwi.append('\n')
 
@@ -458,7 +481,7 @@ def read_espresso_out(fileo):
         for magmoms_index in indexes[_PW_MAGMOM]:
             if image_index < magmoms_index < next_index:
                 magmoms = [
-                    float(mag_line.split()[6]) for mag_line
+                    float(mag_line.split()[5]) for mag_line
                     in pwo_lines[magmoms_index + 1:
                                  magmoms_index + 1 + len(structure)]]
 
