@@ -1,10 +1,9 @@
-from __future__ import print_function
-from os import system,getcwd,chdir 
+from os import system,getcwd
 from os.path import isfile,exists
 from ..molecule import enlarge,molecules,get_neighbors
 from ..data.mdtodata import MDtoData
 from ase.io import read,write
-from ase.io.trajectory import TrajectoryWriter,Trajectory
+from ase.io.trajectory import TrajectoryWriter
 from ase.calculators.singlepoint import SinglePointCalculator
 from ase.units import Ry
 from ase import Atoms
@@ -88,7 +87,6 @@ def Basis():
                   }
     return basis
 
-
 def siesta_traj(label='siesta',batch=10000):
     cwd = getcwd()
     d = MDtoData(structure=label,dft='siesta',direc=cwd,batch=batch)
@@ -96,7 +94,6 @@ def siesta_traj(label='siesta',batch=10000):
     d.close()
     return images
 
-
 def xv2xyz():
     fxv = open('siesta.XV','r')
     fxyz= open('siesta.xyz','w')
@@ -113,7 +110,6 @@ def xv2xyz():
             z = float(line.split()[4])*0.53
             print(elements[line.split()[1]],x,y,z,file=fxyz)
 
-
 def single_point(atoms,id=0,xcf='VDW',xca='DRSLL',basistype='DZP',
                  val={'C':4.0,'H':1.0,'O':6.0,'N':5.0,'F':7.0,'Al':3.0},
                  cpu=4,**kwargs):
@@ -144,8 +140,7 @@ def single_point(atoms,id=0,xcf='VDW',xca='DRSLL',basistype='DZP',
 
 def siesta_opt(atoms=None,label='siesta',ncpu=4,supercell=[1,1,1],
                VariableCell='true',xcf='VDW',xca='DRSLL',basistype='DZP',
-               dt=0.5,T=None,tstep=200,
-               us='F',P=0.0,
+               tstep=200,us='F',P=0.0,
                gen='poscar.gen',**kwargs):
     if exists('siesta.MDE') or exists('siesta.MD_CAR'):
        system('rm siesta.MDE siesta.MD_CAR')
@@ -177,7 +172,6 @@ def siesta_md(atoms=None,label='siesta',gen='poscar.gen',ncpu=4,supercell=[1,1,1
     images = siesta_traj(label=label,batch=tstep)
     return images
 
-
 def run_siesta(ncpu=12):
     system('mpirun -n %d siesta<in.fdf>siesta.out' %ncpu)
 
@@ -207,7 +201,6 @@ def write_siesta_in(Atoms,coord='zmatrix',
 
     m,A  = enlarge(m,cell=cell,fac = fac,supercell=[1,1,1])
     cell = A.get_cell()
-    n_m  = len(m)
 
     r = [0.0 for i in range(3)]
     r[0] = cell[0][0]*cell[0][0]+ cell[0][1]*cell[0][1]+ cell[0][2]*cell[0][2]
@@ -530,7 +523,6 @@ def print_zmatrix(ind,pos,neighors,nneigh,species,atoms_label,table,fdf):
         if link:
            print_zmatrix(a,pos,neighors,nneigh,species,atoms_label,table,fdf)
     
-
 def pseudo_gen(pa='vw',rc_dic=None):
     '''
       Pseudopotential generation
@@ -693,7 +685,7 @@ def get_siesta_cart(label='siesta'):
        fra = (nl-(natom+7)*nframe)/(natom+7)
        print('-  %d frames more than expected, error case            ' %fra)
        exit()
-       
+
     lsp = lines[5].split()
     nsp = [int(l) for l in lsp]
     xs = []
@@ -732,14 +724,42 @@ def get_siesta_cart(label='siesta'):
     cell=np.array([a,b,c])
     his.close()
 
-def get_siesta_fdf(label='siesta'):
-    fin = open(label+'.fdf','r') 
+def parse_fdf_species(fdf='in.fdf'):
+    fin = open(fdf,'r') 
     lines= fin.readlines()
-    fin.close()           # getting informations from input file
-    atom_name = []
+    fin.close()                     # getting informations from input file
 
-    natom = 0
-    ns    = 0
+    natom     = 0
+    ns        = 0
+    spec      = {}
+
+    for i,line in enumerate(lines):
+        l = line.split()
+        if len(l)>0:
+           if l[0] == 'NumberOfSpecies':
+              ns = int(l[1])
+           if l[0] == 'NumberOfAtoms':
+              natom = int(l[1])
+        if ns and natom:
+           break
+
+    for i,line in enumerate(lines):
+        l = line.split()
+        if len(l)>0:
+           if l[0]=='%block':
+              if l[1]=='ChemicalSpeciesLabel' or l[1]=='ChemicalSpecieslabel':
+                 for s in range(ns):
+                     l_ = lines[i+s+1].split()
+                     spec[s+1] = l_[2].split('.')[0]
+    return spec
+
+def parse_fdf(label='siesta',spec={}):
+    fin = open(label+'.fdf','r') 
+    lines= fin.readlines()
+    fin.close()                     # getting informations from input file
+    atom_name = []
+    natom     = 0
+    ns        = 0
 
     for i,line in enumerate(lines):
         l = line.split()
@@ -751,7 +771,6 @@ def get_siesta_fdf(label='siesta'):
         if ns and natom:
            break
 
-    spec      = []
     positions = []
     atom_name = []
     cell      = []
@@ -767,17 +786,29 @@ def get_siesta_fdf(label='siesta'):
               if l[1]=='ChemicalSpeciesLabel' or l[1]=='ChemicalSpecieslabel':
                  for s in range(ns):
                      l_ = lines[i+s+1].split()
-                     spec.append(l_[2].split('.')[0])
+                     spec[s+1] = l_[2].split('.')[0]
               if l[1]=='AtomicCoordinatesAndAtomicSpecies':
                  for p in range(natom):
                      l_ = lines[i+p+1].split()
                      positions.append([float(l_[0]),float(l_[1]),float(l_[2])])
                      # print(spec,int(l_[3])-1)
-                     atom_name.append(spec[int(l_[3])-1])
-
+                     atom_number = int(l_[3])
+                     if atom_number not in spec:
+                        spec[atom_number] = input('Please input symbol of number {:d}: '.format(atom_number))
+                     atom_name.append(spec[atom_number])
+           elif l[0]=='LatticeConstant':
+              if line.find('Ang')>0 or line.find('ang')>0:
+                 unit = 1.0
+              elif line.find('Bohr')>0 or line.find('Bohr')>0:
+                 unit = 0.529177249
+              else:
+                 unit = 1.0
+    cell      = np.array(cell)*unit
+    positions = np.dot(positions,cell)
     A = Atoms(atom_name,positions,cell=cell,pbc=[True,True,True])
-    A.write(label+'.gen')
-    
+    # A.write(label+'.gen')
+    return A
+
 # if __name__ == '__main__':
 #    from emdk import get_structure,emdk
 #    from ase.io import read,write
