@@ -1,17 +1,17 @@
-import matplotlib.pyplot as plt
-from matplotlib.ticker import MultipleLocator
 import numpy as np
 from os import system, getcwd, chdir,listdir
-from os.path import isfile,exists,isdir
+from os.path import isfile
+
+from pyrsistent import v
 from .dft.cpmd import get_lattice
+from .dft.nwchem import get_nw_gradient,out_xyz
 from ase import Atoms
-from ase.io import read,write
 from ase.io.trajectory import TrajectoryWriter,Trajectory
 from .md.gulp import write_gulp_in,get_reaxff_q
 #from .reaxfflib import read_ffield,write_lib
 from .qeq import qeq
 import random
-import pickle
+# import pickle
 # np.set_printoptions(threshold=np.inf) 
 
 
@@ -25,7 +25,6 @@ def rtaper(r,rmin=6.75,rmax=7.5):
     f     = rterm*rd*rd*trm1*r2r
     return  f+r1
 
-
 def taper(r,rmin=0.001,rmax=0.002):
     r1  = np.where(r>rmax,1.0,0.0) # r > rmax then 1 else 0
     rr  = np.where((r>rmin) & (r<rmax),1.0,0.0) # r > rmax then 1 else 0
@@ -36,49 +35,65 @@ def taper(r,rmin=0.001,rmax=0.002):
     f = rterm*rd*rd*trm1*rr
     return  f+r1
 
-
-def get_data(structure='data',direc=None,out=None,
+def get_data(structure='data',direc=None,
              vdwcut=10.0,rcut=None,rcuta=None,
              hbshort=6.5,hblong=7.5,
              dft='nwchem',atoms=None,
              batch=1000,sample='random',
-             minib=100,
-             p=None,spec=None,bonds=None,
-             sort=False,pkl=False,nindex=[]):
-    if (not isfile('data/'+structure + '.pkl')) or (not pkl):
-       t = direc.split('.')
-       if t[-1]=='traj':
-          dft='ase'
-       data = reax_data(structure=structure,direc=direc,
-               vdwcut=vdwcut,
-               rcut=rcut,rcuta=rcuta,
-               hbshort=hbshort,hblong=hblong,
-               dft=dft,atoms=None,
-               batch=batch,minib=minib,sort=sort,
-               p=p,spec=spec,bonds=bonds,
-               nindex=nindex)
-       if not data is None:
-          if pkl:
-             f = open('data/'+structure+'.pkl', 'wb') # open file with write-mode 
-             pickle.dump(data,f)
-             f.close()
-    else:
-       f = open('data/'+structure+'.pkl', 'rb') 
-       data = pickle.load(f)  
-       f.close()
+             minib=100,variable_batch=False,
+             p=None,spec=None,bonds=None,angs=None,
+             tors=None,hbs=None,
+             sort=False,nindex=[]):
+    t = direc.split('.')
+    if t[-1]=='traj':
+       dft='ase'
+    data = reax_data(structure=structure,direc=direc,
+                     sample=sample,
+                     vdwcut=vdwcut,
+                     rcut=rcut,rcuta=rcuta,
+                     hbshort=hbshort,hblong=hblong,
+                     dft=dft,atoms=atoms,
+                     batch=batch,minib=minib,sort=sort,
+                     variable_batch=variable_batch,
+                     p=p,spec=spec,bonds=bonds,angs=angs,
+                     tors=tors,hbs=hbs,
+                     nindex=nindex)
     return data
 
+class Dataset(object):
+  '''Data set to feed the ReaxFF-nn computaional graph'''
+  def __init__(self,dft_energy,forces,rbd,rv,qij,theta,s_ijk,s_jkl,w,rhb,frhb,hbthe):
+      self.dft_energy = dft_energy
+      self.forces     = forces
+      self.rbd        = rbd.transpose()
+      self.rv         = rv
+      self.qij        = qij
+      self.theta      = theta
+      self.s_ijk      = s_ijk
+      self.s_jkl      = s_jkl
+      self.w          = w
+      self.rhb        = rhb
+      self.frhb       = frhb
+      self.hbthe      = hbthe
+
 
 class reax_data(object):
-  """ Collecting datas for mathine learning for bond order potential"""
+  """ Collecting datas for mathine learning for bond order potential ReaxFF or ReaxFF-nn
+      Atribute:
+      --------
+         bond:  int bond table list such as [[1,2],[1,3],...]
+         rbd:   all bonds in the bond listed in "bond" list
+  """
   def __init__(self,structure='cl20mol',botol=0.001,direc=None,
                vdwcut=10.0,rcut=None,rcuta=None,
                hbshort=6.75,hblong=7.5,
                dft='ase',atoms=None,
                batch=1000,minib=100,sample='uniform',
-               p=None,spec=None,bonds=None,
+               p=None,spec=None,bonds=None,angs=None,
+               tors=None,hbs=None,
                sort=False,
                traj=False,
+               variable_batch=False,
                nindex=[]):
       self.sort  = sort
       self.structure = structure
@@ -100,6 +115,9 @@ class reax_data(object):
       self.p         = p
       self.spec      = spec
       self.bonds     = bonds
+      self.angs      = angs
+      self.tors      = tors
+      self.hbs       = hbs
       self.r_cut     = rcut
       self.rcuta     = rcuta
       self.traj      = traj
@@ -132,10 +150,18 @@ class reax_data(object):
             trajonly    = True
       else:
          print('-  Not supported yet!')
+      
+      if variable_batch:
+         if self.nframe>batch:
+            n = self.nframe//batch
+            self.batch = int(self.nframe/n)
+         else:
+            self.batch = self.nframe 
+      else:
+         self.batch = batch
 
-      self.batch = batch  
       self.minib = minib    
-      nni =  len(nindex)   
+      nni        = len(nindex)   # the number of frame alread be collected
 
       random.seed()
       pool = np.arange(self.nframe)
@@ -199,23 +225,24 @@ class reax_data(object):
       self.min_e     = min(self.energy_nw)
       self.max_e     = max(self.energy_nw)
 
-      self.natom_images = self.natom                                    # all atoms include images
-      r_  = max(self.r_cut.values()) # * 2.0
+      # self.natom_images = self.natom                                    # all atoms include images
+      r_  = max(self.r_cut.values())* 2.0
       R_  = np.sqrt(np.sum(self.cell*self.cell,axis=1))
 
-      self.image_mask = [0,0,0]
+      # self.image_mask = [0,0,0]
       for i in range(3):
           if R_[i]<r_:
-             self.image_mask[i] = 1
+             #self.image_mask[i] = 1
+             print('*  Warning: cell length is to small, using images!')
              
-      self.R,self.vr,self.R_image,self.vr_image = self.compute_bond(self.x,self.natom,self.nframe)
+      self.R,self.vr = self.compute_bond(self.x)
       image_rs = self.compute_image(self.vr)                            # vdw interaction images
 
       self.get_table()
-      self.get_bonds(self.R_image)
+      self.get_bonds(self.R)
 
-      self.compute_angle(self.R_image,self.vr_image)
-      self.compute_torsion(self.R_image,self.vr_image)
+      self.compute_angle(self.R,self.vr)
+      self.compute_torsion(self.R,self.vr)
 
       self.compute_hbond(image_rs)
       self.compute_vdw(image_rs)
@@ -228,23 +255,17 @@ class reax_data(object):
       self.get_charge()
       self.get_ecoul(image_rs)
       self.get_eself()
-
-      self.nbe0 = {}
-      for bd in self.bonds:
-          self.nbe0[bd] = 0
-
-      for i in range(self.natom-1):
-          for j in range(i+1,self.natom):
-              bd = self.atom_name[i] + '-' + self.atom_name[j]
-              if bd not in self.bonds:
-                 bd = self.atom_name[j] + '-' + self.atom_name[i]
-              self.nbe0[bd] += 1
+      
+      if variable_batch:
+         # print('-  grouping rvdw ...')
+         self.group_rvdw()
+         self.group_rhb()
 
       # for i,e in enumerate(self.energy_nw):  # new version zpe = old zpe + max_e
       #     self.energy_nw[i] = e - self.max_e  
       print('-  end of gathering datas from directory {:s} ...\n'.format(direc))
 
-  def compute_bond(self,x,natom,nframe):
+  def compute_bond(self,x):
       hfcell    = 0.5 
       u         = np.linalg.inv(self.cell)
       x         = np.array(x)                       #  project to the fractional coordinate
@@ -266,48 +287,13 @@ class reax_data(object):
       
       vr  = np.dot(vr,self.cell) # convert to ordinary coordinate
       R   = np.sqrt(np.sum(vr*vr,axis=3),dtype=np.float32)
-
-      xf_image  = xf
-      images    = [xf]
-
-      if any(self.image_mask):                      # PBC for three difference cases
-         i_ =(-1,2) if self.image_mask[0] else (0,1)
-         j_ =(-1,2) if self.image_mask[1] else (0,1)
-         k_ =(-1,2) if self.image_mask[2] else (0,1)
-         # print(i_,j_,k_)
-         for i in range(i_[0],i_[1]):
-             for j in range(j_[0],j_[1]):
-                 for k in range(k_[0],k_[1]):
-                     if i==0 and j==0 and k==0:
-                        continue
-                     cell = np.array([i,j,k])
-                     images.append(xf_image+cell)
-
-         xf_image = np.concatenate(images,axis=1)   
-         self.natom_images = xf_image.shape[1]
-         # print(self.natom_images)
-         xf_ = np.dot(xf_image,self.cell)
-         # for n,xx in enumerate(xf_[0]):
-         #     print(n,1,end=' ') 
-         #     for x in xx:
-         #         print(x,end=' ') 
-         #     print(' ')
-
-         xj_image   = np.expand_dims(xf_image,axis=1)
-         xi_image   = np.expand_dims(xf_image,axis=2)   
-         vr_image_  = xj_image - xi_image  
-         vr_image   = np.dot(vr_image_,self.cell)     # convert to ordinary coordinate
-         R_image    = np.sqrt(np.sum(vr_image*vr_image,axis=3),dtype=np.float32)
-      else:
-         R_image    = R
-         vr_image   = vr
-      return R,vr,R_image,vr_image
+      return R,vr 
 
   def get_neighbors(self,atoms=None,R=None,rcut=None,rcuta=None):
       table  = [[] for i in range(self.natom)]
       atable = [[] for i in range(self.natom)]
       for i in range(self.natom):
-          for j in range(self.natom_images):
+          for j in range(self.natom):
               if j==i:
                  continue
               i_ = i #% self.natom
@@ -321,7 +307,7 @@ class reax_data(object):
               elif pairr in rcut:
                  key = pairr
               else:
-                 print('-  Error: rcut of bond {:s} not found. '.format(pair))
+                 raise RuntimeError('-  Error: rcut of bond {:s} not found. '.format(pair))
               # print(r,i,j,rcut[key])
               if r<rcut[key]:
                  table[i].append(j)
@@ -333,8 +319,8 @@ class reax_data(object):
       return table,atable
 
   def get_table(self):
-      for nf,X in enumerate(self.x):
-          table,atable = self.get_neighbors(atoms=self.atom_name,R=self.R_image[nf],
+      for nf in range(len(self.x)):
+          table,atable = self.get_neighbors(atoms=self.atom_name,R=self.R[nf],
                                             rcut=self.r_cut,rcuta=self.rcuta)
           if nf==0:
              self.table  = table
@@ -355,10 +341,11 @@ class reax_data(object):
       print('\n')
 
   def get_bonds(self,R):
-      self.bond = []
-      bf        = []
+      self._bond   = []
       self.max_nei = 0
-      # bondpair = []
+      bond_        = {}
+      self.B       = {}
+      self.nbd     = {}
       for i in range(self.natom): 
           if len(self.table[i])>self.max_nei:
              self.max_nei = len(self.table[i])
@@ -369,49 +356,102 @@ class reax_data(object):
                  bn = self.atom_name[i]+'-'+self.atom_name[j_]
                  bnr= self.atom_name[j_]+'-'+self.atom_name[i]
                  if bn in self.bonds:
-                    pair  = [i,j]
-                    pairr = [j,i]
+                    bd = bn
+                    pair  = (i,j)
+                    pairr = (j,i)
                  elif bnr in self.bonds and bnr!=bn:
-                    pair  = [j,i]
-                    pairr = [i,j]
+                    bd = bnr
+                    pair  = (j,i)
+                    pairr = (i,j)
                  else:
-                    print('-  an error case encountered, {:s} not found in bondlist.'.format(bn))
-                    exit()
+                    raise RuntimeError('-  an error case encountered, {:s} not found in bondlist.'.format(bn))
                  # print('-  pair of atom %d & %d, %s' %(i,j,bn),pair)
-                 if (not pair in self.bond) and (not pairr in self.bond):
-                    self.bond.append(pair)
-                    if j>=self.natom:
-                       bf.append([0.5])
-                    else:
-                       bf.append([1.0])
 
-                    # print('-  adding pair of atom %d & %d, name %s' %(i,j,bn),pair)
-      self.bond   = np.array(self.bond,dtype=np.int64)
-      self.nbond  = len(self.bond)
+                 if bd not in bond_:
+                    bond_[bd] = []
+                 if (not pair in bond_[bd]) and (not pairr in bond_[bd]):
+                    bond_[bd].append(pair)
+
+      for bd in self.bonds:
+          st = len(self._bond)
+          if bd in bond_:
+             self._bond.extend(bond_[bd])
+          ed = len(self._bond)
+          self.B[bd]   = (st,ed-st)
+          self.nbd[bd] = ed - st
+          
+      self.nbond  = len(self._bond)
+
+      self.blist  = [] # np.zeros((self.natom,self.max_nei))
+      for i,tab in enumerate(self.table):                       # construct bond table of every atom
+          bl = [[0] for _ in range(self.max_nei)]
+          for n,j in enumerate(tab):
+              bd_ = (i,j)
+              if bd_ not in self._bond:
+                 bd_ = (j,i)
+              nb = self._bond.index(bd_)                     
+              bl[n][0] = nb+1
+          self.blist.append(bl)
+      self.blist = np.array(self.blist)
+      #self.blist.expand_dims(axis=0)
+      
+      self.bond   = np.array(self._bond,dtype=np.int64)
       self.rbd    = R[:,self.bond[:,0],self.bond[:,1]]
-      self.bf     = np.array(bf,dtype=np.float32)
-      #self.bond  = np.mod(self.bond, self.natom)
+      dilink      = np.expand_dims(self.bond[:,0],axis=1)
+      djlink      = np.expand_dims(self.bond[:,1],axis=1)
+      self.dilink,self.djlink = {},{}
+      for bd in bond_:
+          self.dilink[bd] = dilink[self.B[bd][0]:self.B[bd][0]+self.B[bd][1]]
+          self.djlink[bd] = djlink[self.B[bd][0]:self.B[bd][0]+self.B[bd][1]]
 
   def compute_angle(self,R,vr):
-      ang_ind,self.ang_i,self.ang_j,self.ang_k = {},[],[],[]
-      for i in range(self.natom): # atomic energy of i
+      angles,self.A,self.ang_i,self.ang_j,self.ang_k = {},{},[],[],[]
+      self.na = {}
+
+      for i in range(self.natom):
           for n_j,j in enumerate(self.atable[i]):         
               if j >= self.natom or j==i or self.atom_name[j]=='H':
                  continue
-              for n_k,k in enumerate(self.atable[j]):
+              for k in self.atable[j]:
                   if k != i and k!=j:
-                     ang = str(i)+'-'+str(j)+'-'+str(k)
-                     angr= str(k)+'-'+str(j)+'-'+str(i)
-                     if (not ang in ang_ind) and (not angr in ang_ind):
-                        ang_ind[ang]  = True
-                        ang_ind[angr] = True
-                        self.ang_i.append(i)
-                        self.ang_j.append(j)
-                        self.ang_k.append(k)
+                     ang = (i,j,k)
+                     angr= (k,j,i)
+                     an  = self.atom_name[i]+'-' + self.atom_name[j]+'-'+self.atom_name[k]
+                     if an not in self.angs:
+                        an  = self.atom_name[k]+'-' + self.atom_name[j]+'-'+self.atom_name[i]
+                        ang = angr
+                     if an not in angles:
+                        angles[an] = [] 
+                     if (not ang in angles[an]) and (not angr in angles[an]):
+                        angles[an].append(ang)
+                        
 
+      for ang in self.angs:
+          if ang in angles:
+             st= len(self.ang_i)
+             self.ang_i.extend(np.array(angles[ang])[:,0])
+             ed= len(self.ang_i)
+             self.ang_j.extend(np.array(angles[ang])[:,1])
+             self.ang_k.extend(np.array(angles[ang])[:,2])
+             self.A[ang] = (st,ed-st)
+             self.na[ang] = ed - st
       self.nang = len(self.ang_i)
       print('-  number of angles: {:d} ...\n'.format(self.nang))
+      
+      self.abij,self.abjk = [],[]
+      for i in range(self.nang):
+          bij = (self.ang_i[i],self.ang_j[i])
+          if bij not in self._bond:
+             bij = (self.ang_j[i],self.ang_i[i])
+          n = self._bond.index(bij)
+          self.abij.append([n])
 
+          bjk = (self.ang_j[i],self.ang_k[i])
+          if bjk not in self._bond:
+             bjk = (self.ang_k[i],self.ang_j[i])
+          n = self._bond.index(bjk)
+          self.abjk.append([n])
+    
       Rij   = R[:,self.ang_i,self.ang_j]
       Rjk   = R[:,self.ang_j,self.ang_k]
       # Rik = R[:,self.ang_i,self.ang_k]
@@ -428,35 +468,84 @@ class reax_data(object):
 
       self.cos_theta = np.transpose(cos_theta,[1,0])
       self.theta     = np.arccos(self.cos_theta)
-      # self.ang_i     = np.mod(self.ang_i, self.natom)
-      # self.ang_j     = np.mod(self.ang_j, self.natom)
-      # self.ang_k     = np.mod(self.ang_k, self.natom)
 
   def compute_torsion(self,R,vr):
-      # print('-  compute torsion angles ...\n')
-      tor_ind,self.tor_i,self.tor_j,self.tor_k,self.tor_l = {},[],[],[],[]
+      '''  compute torsion angles  '''
+      self.tor_i,self.tor_j,self.tor_k,self.tor_l = [],[],[],[]
+      torsion,self.T,self.nt = {},{},{}
       for i in range(self.natom): # atomic energy of i
           for n_j,j in enumerate(self.atable[i]):   
               if j>= self.natom or j==i or self.atom_name[j]=='H':
                  continue
-              for n_k,k in enumerate(self.atable[j]):
+              for k in self.atable[j]:
                   if k >= self.natom or k == i or k==j or self.atom_name[k]=='H':
                      continue
-                  for n_l,l in enumerate(self.atable[k]):
+                  for l in self.atable[k]:
                       if l==k or l==j or l==i:
                          continue
-                      tor = str(i)+'-'+str(j)+'-'+str(k)+'-'+str(l)
-                      torr= str(l)+'-'+str(k)+'-'+str(j)+'-'+str(i)
-                      if (not tor in tor_ind) and (not torr in tor_ind):
-                         tor_ind[tor] = True
-                         tor_ind[torr] = True
-                         self.tor_i.append(i)
-                         self.tor_j.append(j)
-                         self.tor_k.append(k)
-                         self.tor_l.append(l)
+                      
+                      t1=self.atom_name[i]+'-'+self.atom_name[j]+'-'+self.atom_name[k]+'-'+self.atom_name[l]
+                      t2=self.atom_name[l]+'-'+self.atom_name[k]+'-'+self.atom_name[j]+'-'+self.atom_name[i]
+                      t3=self.atom_name[i]+'-'+self.atom_name[j]+'-'+self.atom_name[k]+'-X'
+                      t4='X-'+self.atom_name[k]+'-'+self.atom_name[j]+'-'+self.atom_name[i]
+                      t5='X-'+self.atom_name[j]+'-'+self.atom_name[k]+'-X'
+                      t6='X-'+self.atom_name[k]+'-'+self.atom_name[j]+'-X'
+                      if t1 in self.tors:
+                         t = t1
+                      elif t2 in self.tors:
+                         t = t2
+                      elif t3 in self.tors:
+                         t = t3
+                      elif t4 in self.tors:
+                         t = t4
+                      elif t5 in self.tors:
+                         t = t5   
+                      elif t6 in self.tors:
+                         t = t6    
+                      else:
+                         raise RuntimeError('-  torsion angle {:s} not in list!'.format(t1))
+                      
+                      if t not in torsion:
+                         torsion[t] = []
+                      tor = (i,j,k,l)
+                      torr= (l,k,j,i)
+                      if (not tor in torsion[t]) and (not torr in torsion[t]):
+                         torsion[t].append(tor)
+      for t in self.tors:
+          st = len(self.tor_i)
+          if t in torsion:
+             self.tor_i.extend(np.array(torsion[t])[:,0])
+             self.tor_j.extend(np.array(torsion[t])[:,1])
+             self.tor_k.extend(np.array(torsion[t])[:,2])
+             self.tor_l.extend(np.array(torsion[t])[:,3])
+          ed = len(self.tor_i)
+          self.T[t] = (st,ed-st)
+          self.nt[t] = ed - st
 
       self.ntor = len(self.tor_i)
       print('-  number of torsion angles: {:d} \n'.format(self.ntor))
+
+      self.tij,self.tjk,self.tkl = [],[],[]
+      for n in range(self.ntor):
+          i = self.tor_i[n]
+          j = self.tor_j[n]
+          k = self.tor_k[n]
+          l = self.tor_l[n]
+          t = (i,j)
+          if t not in self._bond:
+             t = (j,i)
+          self.tij.append([self._bond.index(t)])
+
+          t = (j,k)
+          if t not in self._bond:
+             t = (k,j)
+          self.tjk.append([self._bond.index(t)])
+
+          t = (k,l)
+          if t not in self._bond:
+             t = (l,k)
+          self.tkl.append([self._bond.index(t)])
+
       nb = int(self.batch/self.minib)
       yu = self.batch-nb*self.minib
       if yu>0: nb += 1
@@ -496,7 +585,7 @@ class reax_data(object):
 
           ccijk = np.where(c_ijk>0.99999999,0.0,1.000)
           c2ijk = c_ijk*c_ijk
-          thet_ijk = np.arccos(c_ijk)
+          #thet_ijk = np.arccos(c_ijk)
 
           c     = 1.0-c2ijk
           # print(np.where(np.isinf(c)))
@@ -515,7 +604,7 @@ class reax_data(object):
 
           ccjkl = np.where(c_jkl>0.99999999,0.0,1.0)
           c2jkl = c_jkl*c_jkl
-          thet_jkl = np.arccos(c_jkl)
+          #thet_jkl = np.arccos(c_jkl)
           c = 1.0-c2jkl
           s_jkl = np.sqrt(np.where(c<0.0,0.0,c))
           strm  = np.transpose(s_jkl,[1,0])
@@ -525,7 +614,7 @@ class reax_data(object):
           else:
              self.s_jkl = np.concatenate((self.s_jkl,strm),axis=1)
 
-          c_ijl = (Rij2+Rjl2-Ril2)/(2.0*Rij*Rjl)
+          #c_ijl = (Rij2+Rjl2-Ril2)/(2.0*Rij*Rjl)
 
           c_kjl = (Rjk2+Rjl2-Rkl2)/(2.0*Rjk*Rjl)
           c_kjl = np.where(c_kjl>1.0,1.0,c_kjl)
@@ -567,10 +656,6 @@ class reax_data(object):
              self.cos2w = c2wtm
           else:
              self.cos2w = np.concatenate((self.cos2w,c2wtm),axis=1) 
-      # self.tor_i     = np.mod(self.tor_i, self.natom)  
-      # self.tor_j     = np.mod(self.tor_j, self.natom)
-      # self.tor_k     = np.mod(self.tor_k, self.natom) 
-      # self.tor_l     = np.mod(self.tor_l, self.natom)   
 
   def compute_image(self,vr):
       vr_ = []
@@ -583,8 +668,15 @@ class reax_data(object):
 
   def compute_vdw(self,image_rs):
       vi,vj,vi_p,vj_p,self.vi,self.vj = [],[],[],[],[],[]
+      # self.V = {}
+      # vdws   = {}
+      self.nv  = {}
+      rv_      = {}
       for i in range(self.natom-1):
           for j in range(i+1,self.natom):
+              vn = self.atom_name[i] + '-' + self.atom_name[j] 
+              if vn not in self.bonds:
+                 vn = self.atom_name[j] + '-' + self.atom_name[i] 
               vi_p.append(i)
               vj_p.append(j)
 
@@ -621,12 +713,14 @@ class reax_data(object):
              self.rv = rv_
           else:
              self.rv = np.append(self.rv,rv_,axis=1)
+      
       self.rv = np.transpose(self.rv,[1,0])
+      self.nvb= len(self.rv)
 
   def compute_hbond(self,image_rs):
       self.hb_i,self.hb_j,self.hb_k = [],[],[]
       hb_i,hb_j,hb_k = [],[],[]
-
+      # self.H  = {}
       for i in range(self.natom): 
           if self.atom_name[i]!='H':
              for n_j,j in enumerate(self.atable[i]):   
@@ -697,6 +791,90 @@ class reax_data(object):
          self.hbthe = []
          self.nhb   = 0
 
+  def group_rvdw(self):
+      self.nv   = {}
+      rv        = {}
+      vi        = {}
+      vj        = {}
+      qij       = {}
+      for vb in self.bonds:
+          rv[vb]       = []
+          qij[vb]      = []
+          vi[vb]       = []
+          vj[vb]       = []
+          for i,vi_ in enumerate(self.vi):
+              vj_ = self.vj[i]
+              vn = self.atom_name[vi_]+'-'+self.atom_name[vj_]
+              vn_= self.atom_name[vj_]+'-'+self.atom_name[vi_]
+              if vn==vb or vn_==vb:
+                 rv[vb].append(self.rv[i,:])  # changed here by [:,i]
+                 qij[vb].append(self.q[:,vi_]*self.q[:,vj_]*14.39975840)
+                 vi[vb].append([vi_])
+                 vj[vb].append([vj_])
+          self.nv[vb] = len(vi[vb])
+
+      self.V      = {}
+      self.rv     = []
+      self.vi     = []
+      self.vj     = []
+      self.qij    = []
+      st,ed       = 0,0
+      for vb in self.bonds:
+          if self.nv[vb]>0:
+             st = ed
+             ed = st+self.nv[vb]
+             self.V[vb] = (st,self.nv[vb])
+             self.rv.extend(rv[vb])
+             self.vi.extend(vi[vb])
+             self.vj.extend(vj[vb])
+             self.qij.extend(qij[vb])
+
+  def group_rhb(self):
+      self.nh   = {}
+      rhb       = {}
+      hij       = {}
+      hbthe     = {}
+      frhb      = {}
+
+      for hb in self.hbs:
+          rhb[hb]    = []
+          hij[hb]    = []
+          hbthe[hb]  = []
+          frhb[hb]   = []
+          for i,hi in enumerate(self.hb_i):
+              # hi = self.hb_i[i]
+              hj = self.hb_j[i]
+              hk = self.hb_k[i]
+              hn = self.atom_name[hi]+'-'+self.atom_name[hj]+'-'+self.atom_name[hk]
+              if hn==hb:
+                 bd = self.atom_name[hi]+'-'+self.atom_name[hj]
+                 bd_ = (hi,hj)
+                 if bd not in self.bonds:
+                    bd = self.atom_name[hj]+'-'+self.atom_name[hi]
+                    bd_ = (hj,hi)
+                 ibd = self._bond.index(bd_)
+                 hij[hb].append([ibd])
+                 rhb[hb].append(self.rhb[i,:])  
+                 hbthe[hb].append(self.hbthe[i,:])
+                 frhb[hb].append(self.frhb[i,:])
+          self.nh[hb] = len(rhb[hb])
+
+      self.H      = {}
+      self.hij    = hij
+      self.rhb    = []
+      self.hbthe  = []
+      self.frhb   = []
+      st,ed       = 0,0
+      for hb in self.hbs:
+          if self.nh[hb]>0:
+             st = ed
+             ed = st+self.nh[hb]
+             self.H[hb] = (st,self.nh[hb])
+             # self.hij.extend(hij[hb])
+             self.rhb.extend(rhb[hb])
+             self.hbthe.extend(hbthe[hb])
+             self.frhb.extend(frhb[hb])
+
   def get_gulp_energy(self):
       q,ecoul,eself,evdw = [],[],[],[]
       print('-  get charges from gulp ... \n')
@@ -828,7 +1006,8 @@ class reax_data(object):
 
   def get_ase_data(self,images,trajonly):
       ''' getting data in the ase traj '''
-      self.x=[]
+      self.x         = []
+      self.forces    = []
       self.energy_nw = []
       for i,ind_ in enumerate(self.indexs):
           imag = images[ind_]
@@ -843,9 +1022,16 @@ class reax_data(object):
              e = imag.get_potential_energy()
 
           self.x.append(imag.positions)
+          try:
+             force_ = imag.get_forces()
+          except:
+             force_ = None
+             # print('-  ignoring the forces as are not available.')
+          self.forces.append(force_)
           self.energy_nw.append(e)
       self.energy_nw = np.array(self.energy_nw)
       self.x = np.array(self.x)
+      self.forces = np.array(self.forces)
 
   def get_siesta_energy(self,label='siesta'):
       fe = open(label+'.MDE','r')
@@ -897,8 +1083,8 @@ class reax_data(object):
       nl = len(lines)
       if nl-(self.natom+7)*self.nframe!=0:
          fra = (nl-(self.natom+7)*self.nframe)/(self.natom+7)
-         print('-  %d frames more than expected, error case ... ... ...' %fra)
-         exit()
+         raise RuntimeError('-  %d frames more than expected, error case ... ... ...' %fra)
+
 
       lsp = lines[5].split()
       nsp = [int(l) for l in lsp]
@@ -956,7 +1142,7 @@ class reax_data(object):
 
       nsp = {}
       for s in spec_atoms:
-      	  nsp[s] = len(spec_atoms[s])
+          nsp[s] = len(spec_atoms[s])
 
       iframe = 0
       spec_  = []
@@ -1060,8 +1246,7 @@ class reax_data(object):
       elif isfile('inp-nvt'):
          inp='inp-nvt'
       else: 
-         print('-  error: inp-nvt or inp-nve file not found!')
-         exit() 
+         raise FileNotFoundError('-  error: inp-nvt or inp-nve file not found!')
 
       cell = get_lattice(inp=inp)  # surpose the cell is orthogonal
       self.cell = np.array(cell)
@@ -1084,7 +1269,6 @@ class reax_data(object):
       if self.traj:
          his = TrajectoryWriter(self.structure+'.traj',mode='w')
 
-      ii = 0
       for nf in range(nl):
           x = []
            
