@@ -1,14 +1,10 @@
-#!/usr/bin/env python
-# -*- coding: utf-8 -*-
-from __future__ import print_function
-from os import system #, getcwd, chdir 
 import numpy as np
 from ase import Atoms
 from ase.io import read,write
 # from ase.io.trajectory import TrajectoryWriter
 
 
-def graphene(supercell=[3,3,1],write_=False):
+def graphene(supercell=[3,3,1]):
     radius = 1.420 
     pi = 3.1415926535897932
     h  = 3.40
@@ -37,12 +33,12 @@ def graphene(supercell=[3,3,1],write_=False):
     specs = ['C' for i in range(4*supercell[0]*supercell[1]*supercell[2])]
 
     atoms = Atoms(specs,pos,pbc=[True,True,True],cell=cell)
-    if write_:
-       atoms.write('graphene.gen')
+    # atoms.write('graphene.gen')
     return atoms
 
 
 
 if __name__ == '__main__':
-   graphene(supercell=[1,1,1],write_=True)
+   atoms = graphene(supercell=[1,1,1])
+   atoms.write('graphene.gen')
 
